#include <icd_module_api.h>

/* This is a custom call distribution strategy based on the following algo
 * agents are assignged to 3 priority groups see icd_config/icd_agent.conf-> priority=
 * agent in priority group 1 are are all called first at the same time
 * when the first agent in that groups times out or all members are busy
 * its then tries agents in priority groups 1 and 2
 * when the first agent in that groups times out or all members are busy
 * its then tried agents in priority groups 1,2 and 3
 *
 */
/* public apis */
int icd_module_command_agtpri(int fd, int argc, char **argv);
static icd_plugable_fn *icd_module_get_plugable_fns(icd_caller *that);

/* Private implemenations */
static int module_id = 0;
static char *module_name = "Agent_Priority_Groups";

/* maybe we just use the agent struc by default b4  we have an active member 
 * also might use a macro ICD_PLUGABLE_STANDARD; ...*/
static icd_plugable_fn icd_module_plugable_fns;
static icd_plugable_fn icd_module_agent_plugable_fns;
static icd_plugable_fn icd_module_customer_plugable_fns;
static icd_status init_icd_distributor_agent_priority_groups(icd_distributor *that, char *name, icd_config *data);
static icd_status link_callers_via_pop_customer_ring_agent_priority_groups(icd_distributor *dist, void *extra);
static int icd_module__state_bridge_failed(icd_event *event, void *extra); 
static int icd_module__state_channel_failed(icd_event *event, void *extra);
static int icd_module__state_associate_failed(icd_event *event, void *extra);
static int icd_module__limited_ready_state_on_fail(icd_event *event, void *extra);
static int icd_module__pushback_and_ready_on_fail(icd_event *event, void *extra);


int icd_module_load(icd_config_registry *registry) {

	assert(registry != NULL);

    icd_config_registry__register_ptr(registry, "dist", "agentprioritygroups",
									  init_icd_distributor_agent_priority_groups);

    module_id = icd_event_factory__add_module(module_name);
    if (module_id==0) 
        ast_log(LOG_WARNING, "Unable to register Module Name[%s]",module_name);

    ast_verbose(VERBOSE_PREFIX_3 "Registered ICD Module[%s]!\n",icd_module_strings[module_id]);
 
    return 0;
}

int icd_module_unload(void) {
    /* verify you clean up any mem that was created in this module, 
     * exceptions are objects that are part of caller queues or distributotrs they
     * all have there own clean up
    */
    icd_plugable__clear_fns(&icd_module_plugable_fns);
    icd_plugable__clear_fns(&icd_module_agent_plugable_fns);
    icd_plugable__clear_fns(&icd_module_customer_plugable_fns);
    ast_verbose(VERBOSE_PREFIX_3 "Unloaded ICD Module[[agentprioritygroups]!\n");
	
    return 0;

}

static icd_status init_icd_distributor_agent_priority_groups(icd_distributor *that, char *name, icd_config *data) {
    icd_status result ;
    icd_plugable_fn *plugable_fns;
    char buf[ICD_STRING_LEN];

    assert(that != NULL);
    assert(data != NULL);

    strncpy(that->name,name,sizeof(that->name));
    icd_distributor__set_config_params(that, data);
    icd_distributor__create_lists(that, data);
    icd_list__set_node_insert_func((icd_list *)that->agents, icd_list__insert_ordered,
          icd_member__cmp_priority_order);    
    
    icd_distributor__set_link_callers_fn(that,link_callers_via_pop_customer_ring_agent_priority_groups , NULL);
    /* need to set customization points for Distributors plugable functions */
    icd_distributor__set_plugable_fn_ptr(that, &icd_module_get_plugable_fns);
   
    ICD_MEMSET_ZERO(&icd_module_plugable_fns, sizeof(icd_module_plugable_fns));    
    snprintf(buf, sizeof(buf), "PriorityGroupsDefault");
    result = init_icd_plugable_fns(&icd_module_plugable_fns,buf, data);

    ICD_MEMSET_ZERO(&icd_module_agent_plugable_fns, sizeof(icd_module_agent_plugable_fns));    
    snprintf(buf, sizeof(buf), "PriorityGroupsAgentDefault");
    result = init_icd_plugable_fns(&icd_module_agent_plugable_fns,buf, data);

    ICD_MEMSET_ZERO(&icd_module_customer_plugable_fns, sizeof(icd_module_customer_plugable_fns));    
    snprintf(buf, sizeof(buf), "PriorityGroupsCustomerDefault");
    result = init_icd_plugable_fns(&icd_module_customer_plugable_fns,buf, data);

    
    
    /* Now customize plugable interface points for each of the above function ptr strucs */
    plugable_fns = &icd_module_agent_plugable_fns;
    result = icd_plugable__set_state_call_end_fn(plugable_fns, icd_agent__standard_state_call_end, NULL);
    result = icd_plugable__set_cleanup_caller_fn(plugable_fns, icd_agent__standard_cleanup_caller);

    
/*
    result = icd_plugable__set_state_call_end_fn(plugable_fns, icd_module__standard_state_call_end, NULL);
    result = icd_plugable__set_cleanup_caller_fn(plugable_fns, icd_module__standard_cleanup_caller);
*/
    plugable_fns = &icd_module_customer_plugable_fns;
    result = icd_plugable__set_state_call_end_fn(plugable_fns, icd_customer__standard_state_call_end, NULL);
    result = icd_plugable__set_cleanup_caller_fn(plugable_fns, icd_customer__standard_cleanup_caller);

    result = icd_plugable__set_state_associate_failed_fn(plugable_fns, icd_module__state_associate_failed, NULL); 
    result = icd_plugable__set_state_channel_failed_fn(plugable_fns, icd_module__state_channel_failed, NULL); 
    result = icd_plugable__set_state_bridge_failed_fn(plugable_fns, icd_module__state_bridge_failed, NULL);

   
    icd_distributor__create_thread(that);
    ast_verbose(VERBOSE_PREFIX_3 "ICD Distributor[%s] Initialized !\n", name);
    
    return ICD_SUCCESS;
}

int icd_module_command_agtpri(int fd, int argc, char **argv) {
    static char *help[2] = {"help","agtpri"};

    if(argc >= 2) {
        ast_cli(fd,"\n");
        ast_cli(fd,"\n");
        ast_cli(fd,"ICD Module loaded a icd command interface \n");


        ast_cli(fd,"\n");
        ast_cli(fd,"\n");
    }
    else 
        icd_command_help(fd,2,help);

     return 0;
}


icd_plugable_fn *icd_module_get_plugable_fns(icd_caller *that) {
    icd_plugable_fn *plugable_fns = NULL;
    char *dist_name = NULL;
    assert(that != NULL);


    if (icd_caller__get_active_member(that) == NULL) {
        /* always return our standard plugable interface when no distributor */
        plugable_fns = &icd_module_plugable_fns;    

    } else {
        dist_name= vh_read(icd_distributor__get_params(
                           icd_member__get_distributor(icd_caller__get_active_member(that))),"dist");
        /* sanity check did we get called with our distributor dist="agent.priority.group" 
        if  (strcmp(dist_name, ICD_MODULE) == 0) {
        }
        */
        plugable_fns = icd_plugable_fn_list__fetch_fns(icd_caller__get_plugable_fns_list(that), dist_name);    
        if (plugable_fns == NULL) {
            /* if we fetched the caller specific plugab_fnlist for this dist and found 
             * no caller specfic plugable defined 
             * then we just go with our static local interface for agent or customer
            */
            if (icd_caller__has_role(that, ICD_AGENT_ROLE)) 
                plugable_fns = &icd_module_agent_plugable_fns;
            else
                plugable_fns = &icd_module_customer_plugable_fns;
        }

    }
        
    if (plugable_fns == NULL) {
            ast_log(LOG_ERROR, "Caller %d [%s] has no plugable fn aborting ala crash\n",
            icd_caller__get_id(that), icd_caller__get_name(that));       
    } else {
        if (icd_debug)
           ast_log(LOG_DEBUG, "\nCaller id[%d] [%s] using icd_module_plugable_fns[%s] ready_fn[%p] for Dist[%s]\n", 
                icd_caller__get_id(that), icd_caller__get_name(that),
                icd_plugable__get_name(plugable_fns),
                plugable_fns->state_ready_fn, dist_name);
    } 
    assert(plugable_fns != NULL);
    return plugable_fns;

}


/*
static icd_status icd_module__init_agent(icd_agent *that,icd_config *data); 
static icd_status icd_module__init_customer(icd_customer *that, icd_config *data);

static icd_status icd_module__init_caller(icd_caller *that,icd_config *data) {
    assert(that != NULL);
    icd_caller *caller = (icd_caller *)that;
    icd_plugable_fn *plugable_fns =NULL;
    icd_list_iterator *iter;
    
    return ICD_SUCCESS;
}

static icd_status icd_module__init_agent(icd_agent *that,icd_config *data) {
    assert(that != NULL);

    icd_caller *caller = (icd_caller *)that;
    icd_plugable_fn *plugable_fns =NULL;
    icd_list_iterator *iter;

    
    return ICD_SUCCESS;
}

static icd_status icd_module__init_customer(icd_customer *that, icd_config *data) {
    assert(that != NULL);

    return ICD_SUCCESS;
}        
*/
static icd_status link_callers_via_pop_customer_ring_agent_priority_groups(icd_distributor *dist, void *extra) {
    icd_member *customer_member =NULL;
    icd_member *agent_member = NULL;
    icd_caller *customer_caller = NULL;
    icd_caller *agent_caller = NULL;
    int cust_id;
    int agent_id;
    int first_agent_id = -1;
    icd_status result;

    char *tmp_str;
    int agent_priority = 0;
    char agent_priority_str[3];
    int match_found = 1;
    int match_count = 0;

    assert(dist != NULL);
    assert(dist->customers != NULL);
    assert(dist->agents != NULL);

    if (!icd_member_list__has_members(dist->customers)  || !icd_member_list__has_members(dist->agents)) {
        return ICD_ENOTFOUND;
    }


    customer_member = icd_member_list__pop(dist->customers);
    customer_caller = icd_member__get_caller(customer_member);
    if (customer_member == NULL || customer_caller == NULL) {
        ast_log(LOG_ERROR, "ICD Distributor %s could not retrieve customer from list\n", 
                icd_distributor__get_name(dist));
        return ICD_ERESOURCE;
    }
    tmp_str = icd_caller__get_param(customer_caller, "agent_priority");
    if (tmp_str != NULL) {
        agent_priority = atoi(tmp_str);
    } 
    agent_priority++ ;
    snprintf(agent_priority_str,3,"%d",agent_priority);
    icd_caller__set_param_string(customer_caller, "agent_priority", agent_priority_str);
    tmp_str = icd_caller__get_param(customer_caller, "agent_priority");
    
    result = icd_member__distribute(customer_member);
    if (result != ICD_SUCCESS) {
        /* Some other distributor got to this customer first */
        result = ICD_SUCCESS; /* does this matter */
        return result;
    }
    cust_id = icd_caller__get_id(customer_caller);
    if (icd_debug)
        ast_log(LOG_DEBUG, "ICD AgentPriorityDist %s found customer[%s] agent_priority[%d] from list\n", 
                icd_distributor__get_name(dist),icd_caller__get_name(customer_caller), agent_priority);
   
    while (match_found == 1) {
        agent_member = icd_member_list__pop(dist->agents);
        if (agent_member == NULL ) {
            ast_log(LOG_ERROR, "ICD Distributor %s could not pop agent member from list\n", 
                    icd_distributor__get_name(dist));
            return ICD_ERESOURCE;
        }        
        agent_caller = icd_member__get_caller(agent_member);
        if (agent_caller == NULL) {
            ast_log(LOG_ERROR, "ICD Distributor %s could not cast agent caller from member\n", 
                    icd_distributor__get_name(dist));
            return ICD_ERESOURCE;
        }
        if (icd_caller__get_priority(agent_caller) > agent_priority) {
            /* only assign agents with a priority less than or equal to the last groups of agents 
             * that we attempted to bridge this customer caller
             * if this agent is the wrong priority just stuff him back on the dist and get outa here
            */
            icd_distributor__pushback_agent(dist, agent_member);
            if (match_count == 0) {
                /* found no agents to call, stick customer back on list & either timeout or get new agents */
                icd_distributor__pushback_customer(dist, customer_member);
            }
            ast_log(LOG_WARNING, "ICD AgentPriorityDist[%s] need priority[%d] found Agent[%s] WRONG Priority[%d]\n", 
                icd_distributor__get_name(dist),agent_priority, icd_caller__get_name(agent_caller)
                ,icd_caller__get_priority(agent_caller));
            return ICD_EEXISTS;          
        }
        
        if (icd_debug)
            ast_log(LOG_DEBUG, "ICD AgentPriorityDist %s found Agent Caller[%s] priority[%d] from list\n", 
                icd_distributor__get_name(dist),icd_caller__get_name(agent_caller)
                ,icd_caller__get_priority(agent_caller));

        agent_id = icd_caller__get_id(agent_caller);

        /* Make sure we haven't looped through all agents already */
        if (agent_id == first_agent_id) {
            icd_distributor__pushback_agent(dist, agent_member);
            if (match_count == 0) {
                /* found no agents to call, stick customer back on list & either timeout or get new agents */
                icd_distributor__pushback_customer(dist, customer_member);
            }
            return ICD_ENOTFOUND;
        }
        if (first_agent_id == -1) {
            first_agent_id = agent_id;
        }

        /* ok this guy is good to go */
        result = icd_member__distribute(agent_member);
        if (result != ICD_SUCCESS) {
            return result;
        }

        result = icd_caller__join_callers(customer_caller, agent_caller);

        /* Figure out who the bridger is, and who the bridgee is */
        result = icd_distributor__select_bridger(agent_caller, customer_caller);

        ast_verbose(VERBOSE_PREFIX_3 "Distributor[%s] Link CustomerID[%d] to AgentID[%d]\n", 
            icd_distributor__get_name(dist), cust_id, agent_id );
        if (icd_caller__has_role(customer_caller, ICD_BRIDGER_ROLE)) {
            result = icd_caller__bridge(customer_caller);
        } else if (icd_caller__has_role(agent_caller, ICD_BRIDGER_ROLE)) {
            result = icd_caller__bridge(agent_caller);
        } else {
            ast_log(LOG_ERROR, "ICD Distributor %s found no bridger responsible to bridge call\n", 
                    icd_distributor__get_name(dist));
            icd_distributor__pushback_agent(dist, agent_member);
            icd_distributor__pushback_customer(dist, customer_member);
            return ICD_EGENERAL;
        }

        icd_caller__dump_debug(agent_caller);
        match_count++;        
    } /* while poping agents with priority <= current agent_priority as the customer */
    

   

    icd_caller__dump_debug(customer_caller);
    //icd_caller__dump_debug(agent_caller);

    return ICD_SUCCESS;
}



static int icd_module__state_bridge_failed(icd_event *event, void *extra) {
    icd_caller *that=NULL;
    assert(event != NULL);


    /* Actions for when the bridge failed:
     *   - set state to READY (and thus back into distributor)
     *   - set state to READY if bridge failed < 5 times
     *   - set pushed_back attribute and then set state to READY
     *   - destroy caller
     *
     */
    that = (icd_caller *)icd_event__get_source(event);
    assert(that != NULL);
    icd_caller__set_state_on_associations(that,ICD_CALLER_STATE_BRIDGE_FAILED);

    return icd_module__pushback_and_ready_on_fail(event, extra);

}

static int icd_module__state_channel_failed(icd_event *event, void *extra) {
    assert(event != NULL);

    return icd_module__limited_ready_state_on_fail(event, extra);
}

static int icd_module__state_associate_failed(icd_event *event, void *extra) {
    assert(event != NULL);
    /*
     that = (icd_caller *)icd_event__get_source(event);
     assert(that != NULL);
    */

    return icd_module__pushback_and_ready_on_fail(event, extra);
}

static int icd_module__pushback_and_ready_on_fail(icd_event *event, void *extra) {
    icd_caller *that;
    int *value = 0;

    assert(event != NULL);

    that = icd_event__get_source(event);
    assert(that != NULL);

    /* increment the count of the number of failures */ 
    value = (int *)extra;
    
    //(*value)++;

    icd_caller__set_pushback(that);
    icd_caller__set_state(that, ICD_CALLER_STATE_READY);

    /* No veto allowed here */
    return 0;
}


static int icd_module__limited_ready_state_on_fail(icd_event *event, void *extra) {
    icd_caller *that;
    int *value;

    assert(event != NULL);
    assert(extra != NULL);

    that = icd_event__get_source(event);
    assert(that != NULL);

    value = (int *)extra;

    /*
    TBD - Set up cleanup thread and add to list it keeps It will call icd_caller__clear()
    (*value)++;
    if ((*value) < 5) {
        icd_caller__set_state(that, ICD_CALLER_STATE_READY);
    } else {
    }
    on channel state failed, as of now,
    simply call the standard cleanup caller which will put the agent back to the q
    and customer is hung up with his channel torn down.
    More complex cases are yet to be dealt with
    */
    icd_caller__set_pushback(that);
    /* that->cleanup_caller_fn(that); standard is in icd_agent/customer.c */
    /* No veto allowed here */
    return 0;
}

