
#include <icd_module_api.h>

icd_status link_callers_via_pop_customer_find_agent(icd_distributor *dist, void *extra);

static icd_status init_icd_distributor_find_agent(icd_distributor *that, char *name, icd_config *data) {

    assert(that != NULL);
    assert(data != NULL);
    strncpy(that->name,name,sizeof(that->name));
    icd_distributor__set_config_params(that, data);
    icd_distributor__create_lists(that, data);
    icd_list__set_node_insert_func((icd_list *)that->agents, icd_list__insert_fifo, NULL);
    icd_distributor__set_link_callers_fn(that,link_callers_via_pop_customer_find_agent , NULL);
    icd_distributor__create_thread(that);


    ast_verbose(VERBOSE_PREFIX_3 "Registered ICD Distributor[%s] Initialized !\n", name);
    return ICD_SUCCESS;
}

int icd_module_load(icd_config_registry *registry) {

	assert(registry != NULL);
	icd_config_registry__register_ptr(registry, "dist", "findagent",
									  init_icd_distributor_find_agent);

    icd_config_registry__register_ptr(registry, "dist.link", "popcustfindagent",
            link_callers_via_pop_customer_find_agent);
	
    ast_verbose(VERBOSE_PREFIX_3 "Registered ICD Module[FindAgent]!\n");
    
    return 0;
}

int icd_module_unload(void) {
	/*TODO didnt get this far*/
    ast_verbose(VERBOSE_PREFIX_3 "Unloaded ICD Module[FindAgent]!\n");
	return 0;

}

/* Match the agents target id with the customers id. */
icd_status link_callers_via_pop_customer_find_agent(icd_distributor *dist, void *extra) {
    icd_member *customer = NULL;
    icd_member *agent = NULL;
    icd_caller *customer_caller = NULL;
    icd_caller *agent_caller = NULL;
    icd_member_list *agents;
    int cust_id;
    int agent_id;
    int first_agent_id = -1;
    int agent_target_id;
    int cust_target_id;
    int found_match = 0;
    char *tmp_str;
    icd_list_iterator *iter;
    icd_status result;

    assert(dist != NULL);

    /* For each agent on the list, try to find a matching customer. Return
       when first is found and processed so distributor doesn't get hung up. */
    while (found_match == 0) {
        agents = icd_distributor__get_agent_list(dist);
        agent = icd_member_list__pop(agents);
        agent_caller = icd_member__get_caller(agent);
        if (agent == NULL || agent_caller == NULL) {
            ast_log(LOG_ERROR, "FindAgent Distributor %s could not retrieve agent from list\n", icd_distributor__get_name(dist));
            /* What to do with this pseudo-agent? For now, drop on ground. */
            return ICD_ERESOURCE;
        }

        agent_id = icd_caller__get_id(agent_caller);

        /* Make sure we haven't looped through all agents already */
        if (agent_id == first_agent_id) {
            icd_distributor__pushback_agent(dist, agent);
            return ICD_ENOTFOUND;
        }
        if (first_agent_id == -1) {
            first_agent_id = agent_id;
        }

        /*we need a plugable function to determine what we match from the caller 
         * icd_caller__set_param(agent, "targetid", custid) */
        tmp_str = icd_caller__get_param(agent_caller, "targetid");
        if (tmp_str == NULL) {
            ast_log(LOG_WARNING, "FindAgent Distributor %s reports that agent %s has no target id\n", icd_distributor__get_name(dist), icd_caller__get_name(agent_caller));
            icd_distributor__pushback_agent(dist, agent);
            return ICD_ERESOURCE;
        }
        agent_target_id = atoi(tmp_str);
       
        /* Mark it as in use for now, until we see if we have a match */
        result = icd_member__distribute(agent);
        if (result != ICD_SUCCESS) {
            return result;
        }

        /* Go through all the customers looking for a match */
        iter = icd_distributor__get_customer_iterator(dist);
        while (icd_list_iterator__has_more(iter)) {
            customer = (icd_member *)icd_list_iterator__next(iter);
            customer_caller = icd_member__get_caller(customer);
            if (customer == NULL || customer_caller == NULL) {
                ast_log(LOG_ERROR, "FindAgent Distributor %s could not retrieve customer from list\n", icd_distributor__get_name(dist));
                /* Again, drop the pseudo-customer on the ground */
                continue;
            }
            cust_id = icd_caller__get_id(customer_caller);
            if (cust_id == agent_target_id) {
                found_match = 1;
                icd_distributor__remove_caller(dist, customer_caller);
                break;
            }
        }
        destroy_icd_list_iterator(&iter);
        /* If no matching customer, add back to distributor and try the next one. */
        if (found_match == 0) {
            /* Agent list should be FIFO for FindAgent so that agent puts self on end. */
            icd_caller__set_state(agent_caller, ICD_CALLER_STATE_READY);
        }
    }

    /* This should not be possible, but in case we bugger up the programming... */
    if (found_match == 0) {
        ast_log(LOG_ERROR, "FindAgent Distributor %s in impossible state. No recovery attempted.\n", icd_distributor__get_name(dist));
        return ICD_EGENERAL;
    }    

    /* Double check with warning that the customer target ID matches the agent ID */
    tmp_str = icd_caller__get_param(customer_caller, "targetid");
    if (tmp_str == NULL) {
        ast_log(LOG_WARNING, "FindAgent Distributor %s reports that customer %s has no target id\n", icd_distributor__get_name(dist), icd_caller__get_name(customer_caller));
    }
    cust_target_id = atoi(tmp_str);
    if (cust_target_id != agent_id) {
        ast_log(LOG_WARNING, "FindAgent Distributor %s reports that customer %s target does not match agent %s (though the reverse matches)\n", icd_distributor__get_name(dist), icd_caller__get_name(customer_caller), icd_caller__get_name(agent_caller));
    }

    result = icd_member__distribute(customer);
    if (result != ICD_SUCCESS) {
        icd_caller__set_state(agent_caller, ICD_CALLER_STATE_READY);
        return result;
    }

    result = icd_caller__join_callers(customer_caller, agent_caller);

    /* Figure out who the bridger is, and who the bridgee is */
    result = icd_distributor__select_bridger(agent_caller, customer_caller);

    ast_verbose(VERBOSE_PREFIX_3 "FindAgent Distributor [%s] Link CustomerID[%d] to AgentID[%d]\n", icd_distributor__get_name(dist),cust_id, agent_id );
    if (icd_caller__has_role(customer_caller, ICD_BRIDGER_ROLE)) {
        result = icd_caller__bridge(customer_caller);
    } else if (icd_caller__has_role(agent_caller, ICD_BRIDGER_ROLE)) {
        result = icd_caller__bridge(agent_caller);
    } else {
        ast_log(LOG_ERROR, "FindAgent Distributor %s found no bridger responsible to bridge call\n", icd_distributor__get_name(dist));
        return ICD_EGENERAL;
    }

    return ICD_SUCCESS;
}

